package  
{
	import net.flashpunk.Entity;
	import net.flashpunk.graphics.Image;
	import net.flashpunk.graphics.Spritemap;
	import net.flashpunk.utils.Input;
	import net.flashpunk.utils.Key;
	import net.flashpunk.FP;
	
	/**
	 * ...
	 * @author Chris Zamanillo
	 */
	public class Hero extends Entity 
	{
		[Embed(source = "../assets/graphics/hero.png")] 
		public static const HERO_SPRITE:Class;
		
		public static const HERO_WIDTH:int = 16;
		public static const HERO_HEIGHT:int = 16;
		public static const WALK_SPEED:Number = 80.0;
		public static const GUN_OFFSET:int = 8;
				
		protected var m_currentWeapon:Weapon = null;
		protected var m_spriteMap:Spritemap = null;
				
		public function Hero() 
		{
			type = "hero";
			
			m_spriteMap = new Spritemap(HERO_SPRITE, HERO_WIDTH, HERO_HEIGHT, AnimationDone);
			
			m_spriteMap.add("walk", [0, 1], 5, true);
			m_spriteMap.add("idle", [0], 10, true);
			m_spriteMap.play("idle", false);
			
			graphic = m_spriteMap;
			
			setHitbox(HERO_WIDTH, HERO_HEIGHT);
		}
		
		public function SetWeapon(weapon:Weapon):void
		{
			if (m_currentWeapon != null)
			{
				FP.world.remove(weapon);
			}
			
			m_currentWeapon = weapon;
			FP.world.add(m_currentWeapon);
		}
		
		public function GetWeapon():Weapon { return m_currentWeapon; }
		
		protected function AnimationDone():void
		{
			
		}
		
		override public function update():void 
		{
			var deltaX:Number = 0.0;
			var deltaY:Number = 0.0;
			
			if (m_currentWeapon == null)
			{
				SetWeapon(new MachineGun());
			}
			
			var zombie:Zombie = collide("zombie", x, y) as Zombie;
			var explosion:Explosion = collide("explosion", x, y) as Explosion;
			if (zombie != null && zombie.IsSpawned())
			{
				GameState.DeathReason = "You were eaten by a zombie";
				active = false;
				return;
			}
			else if (explosion != null)
			{
				if (explosion.GetOwner() == "rocket")
				{
					GameState.DeathReason = "You blew off your face\nwith a rocket";
				}
				else if (explosion.GetOwner() == "bomb")
				{
					GameState.DeathReason = "You got a little too close\nto that bomb";
				}
				
				active = false;
				return;
			}
			
			if (Input.check(Key.W) || Input.check(Key.UP))
			{
				deltaY = -WALK_SPEED * FP.elapsed;
			}
			else if (Input.check(Key.S) || Input.check(Key.DOWN))
			{
				deltaY = WALK_SPEED * FP.elapsed;
			}
			
			if (Input.check(Key.A) || Input.check(Key.LEFT))
			{
				deltaX = -WALK_SPEED * FP.elapsed;
			}
			else if (Input.check(Key.D) || Input.check(Key.RIGHT))
			{
				deltaX = WALK_SPEED * FP.elapsed;
			}
			
			if ((x + deltaX) >= Main.SCREEN_WIDTH || (x + deltaX) < 0.0)
			{
				deltaX = 0.0;
			}
			
			if ((y + deltaY) >= Main.SCREEN_HEIGHT || (y + deltaY) < 0.0)
			{
				deltaY = 0.0;
			}
		
			if (deltaX != 0.0 && deltaY != 0.0)
			{
				deltaX *= 0.75;
				deltaY *= 0.75;
			}
			
			var oldX:int = x;
			var oldY:int = y;
			moveBy(deltaX, deltaY, "level", false);
			deltaX = x - oldX;
			deltaY = y - oldY;
						
			if (deltaX != 0.0 || deltaY != 0.0)
			{
				m_spriteMap.play("walk", false);
				m_spriteMap.flipped = deltaX < 0.0;
			}
			else
			{
				m_spriteMap.play("idle", false);
			}
			
			if (Input.mouseDown && m_currentWeapon != null)
			{
				m_spriteMap.flipped = Input.mouseX < x;
				var gunOffsetX:int = m_spriteMap.flipped ? 0 : HERO_WIDTH;
				m_currentWeapon.SpawnBullet(x + gunOffsetX, y + GUN_OFFSET, Input.mouseX, Input.mouseY);
			}
					
			super.update();
		}
	}

}