package  
{
	import net.flashpunk.Sfx;
	import net.flashpunk.World;
	import net.flashpunk.FP;
	import net.flashpunk.graphics.Tilemap;
	import flash.utils.ByteArray;
	
	/**
	 * ...
	 * @author Chris Zamanillo
	 */
	public class Level extends Scene
	{
		public static const ZOMBIE_SPAWN_RATE:Number = 2.0;
		public static const WEAPON_CHANGE_RATE:Number = 10.0;
		public static const TOTAL_WEAPONS:int = 5;
		public static const MAX_CLOUDS:int = 5;
		public static const DIFFICULTY_RATE:Number = 25.0;
				
		protected var m_hero:Hero = null;
		protected var m_zombieSpawnRate:Number = ZOMBIE_SPAWN_RATE;
		protected var m_zombieSpawnTimer:Number = 0.0;
		protected var m_weaponChangeRate:Number = WEAPON_CHANGE_RATE;
		protected var m_weaponChangeTimer:Number = 0.0;
		protected var m_levelGraphics:LevelGraphics = null;
		protected var m_weapons:Array = new Array(TOTAL_WEAPONS);
		protected var m_takeThisSound:Sfx = null;
		protected var m_clouds:Array = new Array(MAX_CLOUDS);
		protected var m_ui:UI = null;
		protected var m_hobo:Hobo = null;
		protected var m_difficultyTimer:Number = 0.0;
		
		[Embed(source = "../assets/sfx/takethis.mp3")] 
		public static const TAKE_THIS_SOUND:Class;	
		
		public function Level(world:World) 
		{
			m_world = world;
			
			m_takeThisSound = new Sfx(TAKE_THIS_SOUND, null);
		}
		
		public override function Initialize():void
		{
			m_done = false;
			m_levelGraphics = new LevelGraphics();
			m_world.add(m_levelGraphics);
			m_weaponChangeTimer = 0.0;
			m_zombieSpawnTimer = 0.0;
			m_difficultyTimer = 0.0;
			
			m_ui = new UI();
			m_world.add(m_ui);
			
			for (var i:int = 0; i < GameState.MAX_ZOMBIES; ++i)
			{
				GameState.Zombies[i] = null;
			}
			
			m_levelGraphics.LoadFromXML();
			
			m_weapons[0] = new Shotgun();
			m_weapons[1] = new MachineGun();
			m_weapons[2] = new RocketLauncher();
			m_weapons[3] = new Flamethrower();
			m_weapons[4] = new BombLauncher();
			
			m_hero = new Hero();
			m_hero.x = Main.SCREEN_WIDTH / 2;
			m_hero.y = Main.SCREEN_HEIGHT / 2;
						
			m_world.add(m_hero);
			GameState.Player = m_hero;
			GameState.Score = 0;
			
			for (i = 0; i < MAX_CLOUDS; ++i)
			{
				var cloud:Cloud = new Cloud();
				cloud.x = Math.random() * Main.SCREEN_WIDTH;
				cloud.y = Math.random() * Main.SCREEN_HEIGHT;
				m_clouds[i] = cloud;
				m_world.add(cloud);
			}
						
			m_hobo = new Hobo();
			m_world.add(m_hobo);
		}
		
		public override function Update():void
		{
			if (m_hero.active == false)
			{
				m_done = true;
				return;
			}
			
			m_difficultyTimer += FP.elapsed;
			if (m_difficultyTimer >= DIFFICULTY_RATE)
			{
				m_difficultyTimer = 0.0;
				m_zombieSpawnRate -= 0.2;
				m_zombieSpawnRate = Math.max(m_zombieSpawnRate, 0.2);
			}
			
			for (var i:int = 0; i < GameState.MAX_ZOMBIES; ++i)
			{
				if (GameState.Zombies[i] != null && GameState.Zombies[i].active == false) 
				{
					GameState.Zombies[i] = null;
				}
			}
			
			m_zombieSpawnTimer += FP.elapsed;
			m_zombieSpawnTimer = Math.min(m_zombieSpawnTimer, m_zombieSpawnRate);
			
			if (m_zombieSpawnTimer >= m_zombieSpawnRate)
			{
				SpawnZombie();
				m_zombieSpawnTimer = 0.0;
			}
			
			m_weaponChangeTimer += FP.elapsed;
			m_weaponChangeTimer = Math.min(m_weaponChangeTimer, m_weaponChangeRate);
			
			if (m_weaponChangeTimer >= m_weaponChangeRate)
			{
				ChangeWeapon();
				m_weaponChangeTimer = 0.0;
			}
			
			for (i = 0; i < MAX_CLOUDS; ++i)
			{
				if (m_clouds[i].IsCloudGone())
				{
					m_world.remove(m_clouds[i]);
					
					var cloud:Cloud = new Cloud();
					cloud.x = Main.SCREEN_WIDTH;
					cloud.y = Math.random() * Main.SCREEN_HEIGHT;
					m_clouds[i] = cloud;
					m_world.add(cloud);
				}
			}
		}
		
		public function SpawnZombie():void
		{
			for (var i:int = 0; i < GameState.MAX_ZOMBIES; ++i)
			{
				if (GameState.Zombies[i] == null)
				{
					var zombie:Zombie = new StandardZombie();
			
					do
					{
						zombie.x = Math.floor(Math.random() * (Main.SCREEN_WIDTH / LevelGraphics.LEVEL_GRID_SIZE)) * LevelGraphics.LEVEL_GRID_SIZE;
						zombie.y = Math.floor(Math.random() * (Main.SCREEN_HEIGHT / LevelGraphics.LEVEL_GRID_SIZE)) * LevelGraphics.LEVEL_GRID_SIZE;
					} while (zombie.collide("level", zombie.x, zombie.y) != null);
										
					GameState.Zombies[i] = zombie;
					FP.world.add(zombie);
					break;
				}
			}
		}
		
		public function ChangeWeapon():void
		{
			if (m_hero.active == false)
			{
				return;
			}
			
			var weaponIdx:int;
			do
			{
				weaponIdx = Math.floor(Math.random() * TOTAL_WEAPONS);
			} while (m_weapons[weaponIdx].GetName() == m_hero.GetWeapon().GetName())
			
			m_hero.SetWeapon(m_weapons[weaponIdx]);
			m_takeThisSound.play();
			m_ui.ShowDialog(m_weapons[weaponIdx].GetName());
			m_hobo.Run();
		}
	}

}