package  
{
	import net.flashpunk.graphics.Graphiclist;
	import net.flashpunk.graphics.Spritemap;
	import net.flashpunk.FP;
	import net.flashpunk.graphics.Particle;
	import net.flashpunk.graphics.Emitter;
	import net.flashpunk.Sfx;
	/**
	 * ...
	 * @author Chris Zamanillo
	 */
	public class Rocket extends Bullet 
	{
		[Embed(source = "../assets/graphics/rocket.png")] 
		public static const ROCKET_SPRITE:Class;
		
		[Embed(source = "../assets/graphics/smoke.png")] 
		public static const SMOKE_SPRITE:Class;
		
		public static const BULLET_SPEED:Number = 200.0;
		public static const SMOKE_RATE:Number = 0.005;
		public static const DAMAGE:int = 5;
		
		protected var m_velX:Number = 0.0;
		protected var m_velY:Number = 0.0;
		protected var m_spriteMap:Spritemap = null;
		protected var m_smokeEmitter:Emitter = null;
		protected var m_smokeTimer:Number = 0;
		
		public function Rocket(velX:Number, velY:Number) 
		{
			m_damage = DAMAGE;
			m_spriteMap = new Spritemap(ROCKET_SPRITE, 8, 8);
			
			m_velX = velX;
			m_velY = velY;
			
			var vecUp:Vector2D = new Vector2D(0, -1);
			var velocity:Vector2D = new Vector2D(velX, velY);
			VMath.normalize(velocity);
			var dot:Number = vecUp.dotProd(velocity);
			
			m_spriteMap.angle = Math.acos(dot) * (180.0 / Math.PI);
			if (velX > 0.0)
			{
				m_spriteMap.angle = -m_spriteMap.angle;
			}
			m_spriteMap.centerOrigin();
			setHitbox(8, 8);
			
			m_smokeEmitter = new Emitter(SMOKE_SPRITE, 8, 8);
			m_smokeEmitter.newType("smoke", [0, 1, 2, 3]);
			m_smokeEmitter.setAlpha("smoke", 0.5, 0);
			m_smokeEmitter.setMotion("smoke", m_spriteMap.angle, 0, 0.01, 1, 1, 1);
			m_smokeEmitter.relative = false;
			
			graphic = new Graphiclist(m_spriteMap, m_smokeEmitter);
		}
		
		override public function update():void 
		{
			m_smokeTimer += FP.elapsed;
			m_smokeTimer = Math.min(SMOKE_RATE, m_smokeTimer);
			if (m_smokeTimer >= SMOKE_RATE)
			{
				m_smokeEmitter.emit("smoke", x, y);
				m_smokeTimer = 0.0;
			}
			
			x += m_velX * FP.elapsed;
			y += m_velY * FP.elapsed;
			
			if (collide("level", x, y))
			{
				FP.world.remove(this);
				
				var explosion:Explosion = new Explosion();
				FP.world.add(explosion);
				explosion.Explode(x, y);
			}
						
			super.update();
		}
	}

}