package  
{
	import flash.geom.ColorTransform;
	import net.flashpunk.Entity;
	import net.flashpunk.FP;
	import net.flashpunk.graphics.Spritemap;
	import net.flashpunk.Sfx;
	import net.flashpunk.tweens.misc.ColorTween;
	import net.flashpunk.Tween;
	
	/**
	 * ...
	 * @author Chris Zamanillo
	 */
	public class Zombie extends Entity 
	{
		protected var m_walkSpeed:Number = 0.0;
		protected var m_spriteMap:Spritemap = null;
		protected var m_spawned:Boolean = false;
		protected var m_health:int = 0;
		protected var m_colorTween:ColorTween = null;
		protected var m_riseSound:Sfx = null;
		protected var m_hurtSound:Sfx = null;
		protected var m_scoreValue:int = 0;
		
		[Embed(source = "../assets/sfx/rise.mp3")] 
		public static const RISE_SOUND:Class;
		
		[Embed(source = "../assets/sfx/hurt.mp3")] 
		public static const HURT_SOUND:Class;
		
		public function Zombie() 
		{
			type = "zombie";
			
			m_colorTween = new ColorTween(null, Tween.PERSIST);
			m_colorTween.color = 0xffffff;
			addTween(m_colorTween, false);
			
			m_riseSound = new Sfx(RISE_SOUND, null);
			m_riseSound.play(0.3);
			
			m_hurtSound = new Sfx(HURT_SOUND, null);
		}
		
		public function IsSpawned():Boolean { return m_spawned; }
		
		public virtual function Die():void
		{
			
		}
		
		override public function update():void 
		{
			m_spriteMap.color = m_colorTween.color;

			if(m_spawned)
			{
				var bullet:Bullet = collide("bullet", x, y) as Bullet;
				if (bullet != null)
				{
					m_health -= bullet.GetDamage();
					
					if (m_health <= 0)
					{
						active = false;
						FP.world.remove(this);
						var gibs:Gibs = new Gibs();
						FP.world.add(gibs);
						gibs.BloodSpray(x + (m_spriteMap.width / 2), y + (m_spriteMap.height / 2));
						
						GameState.Score += m_scoreValue;
					}
					else
					{
						m_colorTween.tween(0.1, 0xff0000, 0xffffff)
						m_colorTween.start();
					}
					
					if (bullet is Rocket)
					{
						var rocket:Rocket = bullet as Rocket;
						var rocketExplosion:Explosion = new Explosion();
						FP.world.add(rocketExplosion);
						rocketExplosion.Explode(rocket.x, rocket.y);
					}
					
					m_hurtSound.play();
					
					if (bullet.RemoveOnHit())
					{
						FP.world.remove(bullet);
					}
				}
				
				// GHETTO!
				var explosion:Explosion = collide("explosion", x, y) as Explosion;
				if (explosion != null)
				{
					m_health -= Explosion.DAMAGE;
					
					if (m_health <= 0)
					{
						active = false;
						FP.world.remove(this);
						var gibs2:Gibs = new Gibs();
						FP.world.add(gibs2);
						gibs2.BloodSpray(x + (m_spriteMap.width / 2), y + (m_spriteMap.height / 2));
						
						GameState.Score += m_scoreValue;
					}
					else
					{
						m_colorTween.tween(0.1, 0xff0000, 0xffffff)
						m_colorTween.start();
					}
					
					m_hurtSound.play();
				}
						
				var zombiePos:Vector2D = new Vector2D(x, y);
				var playerPos:Vector2D = new Vector2D(GameState.Player.x, GameState.Player.y);
				
				VMath.subtract(playerPos, zombiePos);
				VMath.normalize(playerPos);
				VMath.multiply(playerPos, m_walkSpeed);
				
				var deltaX:Number = playerPos.x * FP.elapsed;
				var deltaY:Number = playerPos.y * FP.elapsed;
				
				var oldX:int = x;
				var oldY:int = y;
				moveBy(deltaX, deltaY, "level", false);
				
				if (oldX != Math.floor(x))
				{
					m_spriteMap.flipped = oldX > x;
				}
			}
			
			super.update();
		}
	}

}